/*
 *    Compaq Hot Plug Controller Graphical User Interface 
 *    Copyright 2000, 2001 Compaq Computer Corporation
 *    All rights reserved.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, GOOD TITLE or
 *    NON INFRINGEMENT.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Please send all questions or concerns to linuxhotplug@compaq.com
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "commonheader.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "tree.h"

// callbacks for menu items
static GnomeUIInfo file_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Exit"),
    NULL,
    on_exit_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo operations_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Add Adapter"),
    NULL,
    on_add_adapter_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "hotplug/card_insert-small.xpm",
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Remove Adapter"),
    NULL,
    on_remove_adapter_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "hotplug/card_remove-small.xpm",
    0, 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Insert Driver"),
    NULL,
    on_insert_driver_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REDO,
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Remove Driver"),
    NULL,
    on_remove_driver_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_UNDO,
    0, 0, NULL
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Enable Attention Status"),
    NULL,
    on_enable_attention_status_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "hotplug/yellow-small.xpm",
    0, 0, NULL
  },
  {
    GNOME_APP_UI_ITEM, N_("Disable Attention Status"),
    NULL,
    on_disable_attention_status_activate, NULL, NULL,
    GNOME_APP_PIXMAP_FILENAME, "hotplug/disable-small.xpm",
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo view_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("Refresh"),
    NULL,
    on_refresh_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_REFRESH,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] =
{
  {
    GNOME_APP_UI_ITEM, N_("About..."),
    NULL,
    on_about_activate, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT,
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] =
{
  {
    GNOME_APP_UI_SUBTREE, N_("File"),
    NULL,
    file_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, N_("File"),
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("Operations"),
    NULL,
    operations_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, N_("Operations"),
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("View"),
    NULL,
    view_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, N_("View"),
    0, 0, NULL
  },
  {
    GNOME_APP_UI_SUBTREE, N_("Help"),
    NULL,
    help_menu_uiinfo, NULL, NULL,
    GNOME_APP_PIXMAP_NONE, N_("Help"),
    0, 0, NULL
  },
  GNOMEUIINFO_END
};

// nullify global variables
GtkWidget* g_add_adapter_button = NULL;
GtkWidget* g_add_adapter_menu_item = NULL;
GtkWidget* g_remove_adapter_button = NULL;
GtkWidget* g_remove_adapter_menu_item = NULL;
GtkWidget* g_insert_driver_button = NULL;
GtkWidget* g_insert_driver_menu_item = NULL;
GtkWidget* g_remove_driver_button = NULL;
GtkWidget* g_remove_driver_menu_item = NULL;
GtkWidget* g_enable_att_status_button = NULL;
GtkWidget* g_enable_att_status_menu_item = NULL;
GtkWidget* g_disable_att_status_button = NULL;
GtkWidget* g_disable_att_status_menu_item = NULL;

GtkWidget*
create_window (void)
{
  GtkWidget *window;
  GtkWidget *vbox1;
  GtkWidget *menubar;
  GtkWidget *toolbar;
  GtkWidget *tmp_toolbar_icon;
  GtkWidget *Add;
  GtkWidget *Remove;
  GtkWidget *label5;
  GtkWidget *Insert;
  GtkWidget *Remove2;
  GtkWidget *label6;
  GtkWidget *Enable;
  GtkWidget *Disable;
  GtkWidget *label7;
  GtkWidget *Help;
  GtkWidget *hpaned1;
  GtkWidget *scrolledwindow1;
  GtkWidget *tree;
  GtkWidget *label8;
  GtkWidget *scrolledwindow2;
  GtkWidget *clist;
  GtkWidget *label9;
  GtkWidget *label10;
  GtkWidget *statusbar;

  allocate_colors();
  
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window), "window", window);
  gtk_window_set_title (GTK_WINDOW (window), _("Compaq PCI Hot Plug for Linux"));
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size (GTK_WINDOW (window), 700, 500);
  g_window = window;

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window), vbox1);

  menubar = gtk_menu_bar_new ();
  gtk_widget_ref (menubar);
  gtk_object_set_data_full (GTK_OBJECT (window), "menubar", menubar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (menubar);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar, FALSE, FALSE, 0);
  gnome_app_fill_menu (GTK_MENU_SHELL (menubar), menubar_uiinfo,
                       NULL, FALSE, 0);

  gtk_widget_ref (menubar_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "file",
                            menubar_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (file_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "exit",
                            file_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "operations",
                            menubar_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  g_add_adapter_menu_item = operations_menu_uiinfo[0].widget;
  gtk_widget_ref (operations_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "add_adapter",
                            operations_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  g_remove_adapter_menu_item = operations_menu_uiinfo[1].widget;
  gtk_widget_ref (operations_menu_uiinfo[1].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "remove_adapter",
                            operations_menu_uiinfo[1].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (operations_menu_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "separator1",
                            operations_menu_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  g_insert_driver_menu_item = operations_menu_uiinfo[3].widget;
  gtk_widget_ref (operations_menu_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "insert_driver",
                            operations_menu_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  g_remove_driver_menu_item = operations_menu_uiinfo[4].widget;
  gtk_widget_ref (operations_menu_uiinfo[4].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "remove_driver",
                            operations_menu_uiinfo[4].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (operations_menu_uiinfo[5].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "separator",
                            operations_menu_uiinfo[5].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  g_enable_att_status_menu_item = operations_menu_uiinfo[6].widget;
  gtk_widget_ref (operations_menu_uiinfo[6].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "enable_attention_status",
                            operations_menu_uiinfo[6].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  g_disable_att_status_menu_item = operations_menu_uiinfo[7].widget;
  gtk_widget_ref (operations_menu_uiinfo[7].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "disable_attention_status",
                            operations_menu_uiinfo[7].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar_uiinfo[2].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "view",
                            menubar_uiinfo[2].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (view_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "refresh",
                            view_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (menubar_uiinfo[3].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "help",
                            menubar_uiinfo[3].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  gtk_widget_ref (help_menu_uiinfo[0].widget);
  gtk_object_set_data_full (GTK_OBJECT (window), "about",
                            help_menu_uiinfo[0].widget,
                            (GtkDestroyNotify) gtk_widget_unref);

  toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
  gtk_widget_ref (toolbar);
  gtk_object_set_data_full (GTK_OBJECT (window), "toolbar", toolbar,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar);
  gtk_box_pack_start (GTK_BOX (vbox1), toolbar, FALSE, FALSE, 0);

  tmp_toolbar_icon = create_pixmap (window, "hotplug/card_insert.xpm", TRUE);
  g_add_adapter_button = Add = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Add"),
                                _("Add Adapter"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (Add);
  gtk_object_set_data_full (GTK_OBJECT (window), "Add", Add,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Add);

  tmp_toolbar_icon = create_pixmap (window, "hotplug/card_remove.xpm", TRUE);
  g_remove_adapter_button = Remove = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Remove"),
                                _("Remove Adapter"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (Remove);
  gtk_object_set_data_full (GTK_OBJECT (window), "Remove", Remove,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Remove);

  label5 = gtk_label_new ("");
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (window), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), label5, NULL, NULL);
  gtk_widget_set_usize (label5, 20, -2);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (window, GNOME_STOCK_PIXMAP_REDO);
  g_insert_driver_button = Insert = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Insert"),
                                _("Insert Driver"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (Insert);
  gtk_object_set_data_full (GTK_OBJECT (window), "Insert", Insert,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Insert);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (window, GNOME_STOCK_PIXMAP_UNDO);
  g_remove_driver_button = Remove2 = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Remove"),
                                _("Remove Driver"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (Remove2);
  gtk_object_set_data_full (GTK_OBJECT (window), "Remove2", Remove2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Remove2);

  label6 = gtk_label_new ("");
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (window), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), label6, NULL, NULL);
  gtk_widget_set_usize (label6, 20, -2);

  tmp_toolbar_icon = create_pixmap (window, "hotplug/yellow_bullet_icon.xpm", TRUE);
  g_enable_att_status_button = Enable = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Enable"),
                                _("Enable Attention Status"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (Enable);
  gtk_object_set_data_full (GTK_OBJECT (window), "Enable", Enable,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Enable);

  tmp_toolbar_icon = create_pixmap (window, "hotplug/disable_icon.xpm", TRUE);
  g_disable_att_status_button = Disable = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Disable"),
                                _("Disable Attention Status"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (Disable);
  gtk_object_set_data_full (GTK_OBJECT (window), "Disable", Disable,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Disable);

  label7 = gtk_label_new ("");
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (window), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), label7, NULL, NULL);
  gtk_widget_set_usize (label7, 20, -2);

  tmp_toolbar_icon = gnome_stock_pixmap_widget (window, GNOME_STOCK_PIXMAP_HELP);
  Help = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Help"),
                                _("Help"), NULL,
                                tmp_toolbar_icon, NULL, NULL);
  gtk_widget_ref (Help);
  gtk_object_set_data_full (GTK_OBJECT (window), "Help", Help,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (Help);

  hpaned1 = gtk_hpaned_new ();
  gtk_widget_ref (hpaned1);
  gtk_object_set_data_full (GTK_OBJECT (window), "hpaned1", hpaned1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hpaned1);
  gtk_box_pack_start (GTK_BOX (vbox1), hpaned1, TRUE, TRUE, 0);
  gtk_paned_set_position (GTK_PANED (hpaned1), 200);
  gtk_paned_set_handle_size (GTK_PANED(hpaned1), 10);
  gtk_paned_set_gutter_size (GTK_PANED(hpaned1), 4);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (hpaned1), scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  tree = gtk_ctree_new (1, 0);
  gtk_widget_ref (tree);
  gtk_object_set_data_full (GTK_OBJECT (window), "tree", tree,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (tree);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), tree);
  gtk_clist_set_column_width (GTK_CLIST (tree), 0, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (tree));
  gtk_ctree_set_expander_style((GtkCTree*)tree, GTK_CTREE_EXPANDER_NONE);
  gtk_ctree_set_show_stub((GtkCTree*)tree, TRUE);
  g_tree = (GtkCTree*) tree;
 
  label8 = gtk_label_new (_("label8"));
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (window), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_clist_set_column_widget (GTK_CLIST (tree), 0, label8);

  scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_ref (scrolledwindow2);
  gtk_object_set_data_full (GTK_OBJECT (window), "scrolledwindow2", scrolledwindow2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow2);
  gtk_container_add (GTK_CONTAINER (hpaned1), scrolledwindow2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  g_table = clist = gtk_clist_new (2);
  gtk_widget_ref (clist);
  gtk_object_set_data_full (GTK_OBJECT (window), "clist", clist,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (clist);
  gtk_container_add (GTK_CONTAINER (scrolledwindow2), clist);
  gtk_clist_set_column_width (GTK_CLIST (clist), 0, 175);
  gtk_clist_set_column_width (GTK_CLIST (clist), 1, 80);
  gtk_clist_column_titles_hide (GTK_CLIST (clist));

  label9 = gtk_label_new (_("label9"));
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (window), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 0, label9);

  label10 = gtk_label_new (_("label10"));
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (window), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_clist_set_column_widget (GTK_CLIST (clist), 1, label10);
  
  g_statusbar = statusbar = gtk_statusbar_new ();
  gtk_widget_ref (statusbar);
  gtk_object_set_data_full (GTK_OBJECT (window), "statusbar", statusbar,
                            (GtkDestroyNotify) gtk_widget_unref);

  context_id = gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "Linux Hot Plug Driver");

  gtk_widget_show (statusbar);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar, FALSE, FALSE, 0);

  // callback for selecting a tree row
  gtk_signal_connect(GTK_OBJECT(tree), "tree_select_row",
		     GTK_SIGNAL_FUNC(tree_row_selected),
		     NULL);
  
  // callback for selecting a table row
  gtk_signal_connect(GTK_OBJECT(clist), "select_row",
		     GTK_SIGNAL_FUNC(table_row_selected),
		     NULL);
  
  // callbacks for toolbar buttons
  gtk_signal_connect_object (GTK_OBJECT(Add), "clicked",
                      GTK_SIGNAL_FUNC(on_add_adapter_activate),
                      (gpointer) clist);

  gtk_signal_connect_object (GTK_OBJECT(Remove), "clicked",
                      GTK_SIGNAL_FUNC(on_remove_adapter_activate),
                      (gpointer) clist);

  gtk_signal_connect_object (GTK_OBJECT(Insert), "clicked",
                      GTK_SIGNAL_FUNC(on_insert_driver_activate),
                      (gpointer) clist);

  gtk_signal_connect_object (GTK_OBJECT(Remove2), "clicked",
                      GTK_SIGNAL_FUNC(on_remove_driver_activate),
                      (gpointer) clist);

  gtk_signal_connect_object (GTK_OBJECT(Enable), "clicked",
                      GTK_SIGNAL_FUNC(on_enable_attention_status_activate),
                      (gpointer) clist);

  gtk_signal_connect_object (GTK_OBJECT(Disable), "clicked",
                      GTK_SIGNAL_FUNC(on_disable_attention_status_activate),
                      (gpointer) clist);

  gtk_signal_connect_object (GTK_OBJECT(Help), "clicked",
                      GTK_SIGNAL_FUNC(on_about_activate),
                      (gpointer) clist);

  return window;
}
