/*
 *    Compaq Hot Plug Controller Graphical User Interface 
 *    Copyright 2000, 2001 Compaq Computer Corporation
 *    All rights reserved.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, GOOD TITLE or
 *    NON INFRINGEMENT.  See the GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    Please send all questions or concerns to linuxhotplug@compaq.com
 */

#ifndef _TREE_H
#define _TREE_H

#include <stdlib.h>

#define MAXSLOTS	15

struct pci_tree_node_struct;

// prototypes for function pointer calls
typedef void (*fn_set_adapter)(int function, GtkCTree* tree, GtkCTreeNode* node, GtkWidget* table, GtkWidget* status_bar, struct pci_tree_node_struct* user_data);
typedef void (*fn_set_driver)(int function, GtkCTree* tree, GtkCTreeNode* node, GtkWidget* table, GtkWidget* status_bar, struct pci_tree_node_struct* user_data);
typedef void (*fn_set_attention_status)(int function, GtkCTree* tree, GtkCTreeNode* node, GtkWidget* table, GtkWidget* status_bar, struct pci_tree_node_struct* user_data);
typedef void (*fn_build_table_data)(GtkWidget* table, struct pci_tree_node_struct* p_node);
typedef void (*fn_render_node)(GtkCTree* tree, GtkCTreeNode* node, struct pci_tree_node_struct* user_data);

typedef struct pci_tree_node_struct {
	int type;
	void* node_info;
	char proc_path[MAXLEN];
	GtkCTreeNode* node;
	struct pci_tree_node_struct* next;
	fn_set_driver set_driver;
	fn_set_adapter set_adapter;
	fn_render_node render_node;
	fn_build_table_data build_table_data;
	fn_set_attention_status set_attention_status;
} pci_tree_node;

void allocate_colors();
void load_pixmaps(GtkCTree* tree);
void walk_top_layer(GtkCTree* tree, char* curr_dir);
void build_from_slot(GtkCTree* tree, int index);
void remove_node(GtkCTree* tree, GtkCTreeNode* node, gpointer data);
GtkCTreeNode* get_parent(GtkCTreeNode* node);
pci_tree_node* ctree_partner(GtkCTree* tree, GtkCTreeNode* node, int* priority_array);
pci_tree_node* ctree_partner_ex(GtkCTree* tree, GtkCTreeNode* node, int* priority_array, int bridge_preferred);
pci_tree_node* pci_partner(pci_tree_node* node, int* priority_array);
void set_back_pointer(GtkCTree* tree, GtkCTreeNode* node, gpointer data);
void render_node(GtkCTree* tree, GtkCTreeNode* node, gpointer data);
void render_node_from_slot(GtkCTree* tree, GtkCTreeNode* node, pci_tree_node* user_data);
void parse_names(char* device_name, char* driver_name, char* device, char* driver);
void render_node_from_function(GtkCTree* tree, GtkCTreeNode* node, pci_tree_node* user_data);
int number_of_children(GtkCTreeNode* node);
void render_node_from_adapter(GtkCTree* tree, GtkCTreeNode* node, pci_tree_node* user_data);

extern GtkCTreeNode* root_nodes[MAXSLOTS];
extern char root_paths[MAXSLOTS][MAXLEN];
extern int root_paths_size;

#endif
